using System;
using System.IO;
using System.Configuration;
using System.Xml;
using System.Web;
using System.Collections.Generic;
using System.Text;
using System.Web.Caching;
using System.Threading;

namespace Subgurim.Controles.SSM
{

    /// <summary>
    /// Summary description for SiteMapBLL
    /// </summary>
    sealed public class SubgurimSiteMapBLL
    {
        private const string clave = "SubgurimSiteMapCache";
        private const string claveXMLized = "SubgurimSiteMapCache_XMLized";
        private const string claveaccessToken = "SubgurimSiteMapCache_accessToken";

        public SubgurimSiteMapBLL()
        {
            //
            // TODO: Add constructor logic here
            //
        }
        #region Show

        #region SiteMapCache

        /// <summary>
        /// ltima vez que el XML fue guardado en su XML
        /// Last time that the XML was saved on his XML
        /// </summary>
        private static DateTime lastXMLized
        {
            get
            {
                HttpApplicationState app = HttpContext.Current.Application;
                object last = app[claveXMLized];
                if (last == null)
                {
                    last = DateTime.UtcNow;
                    app[claveXMLized] = last;
                }
                return (DateTime)last;
            }
            set
            {
                HttpContext.Current.Application.Lock();
                HttpContext.Current.Application[claveXMLized] = value;
                HttpContext.Current.Application.UnLock();
            }
        }

        private static bool accessToken
        {
            get
            {
                HttpApplicationState app = HttpContext.Current.Application;
                object access = app[claveaccessToken];
                if (access == null)
                {
                    access = true;
                    app[claveaccessToken] = access;
                }
                return (bool)access;
            }
            set
            {
                HttpContext.Current.Application.Lock();
                HttpContext.Current.Application[claveaccessToken] = value;
                HttpContext.Current.Application.UnLock();
            }
        }

        /// <summary>
        /// Recoge el SiteMap
        /// Takes the SiteMap
        /// </summary>
        /// <returns></returns>
        private static Dictionary<string, SubgurimSiteMapBase> SiteMaps
        {
            get
            {
                object SiteMapCache = HttpContext.Current.Cache.Get(clave);
                if (null == SiteMapCache)
                {
                    SiteMapCache = getSiteMapXML();
                    HttpContext.Current.Cache.Insert(clave, SiteMapCache, null, DateTime.Now.AddMonths(12), TimeSpan.Zero);
                }


                return (Dictionary<string, SubgurimSiteMapBase>)SiteMapCache;
            }
            set
            {
                HttpContext.Current.Cache[clave] = value;

            }
        }

        /*
        /// <summary>
        /// Mtodo llamado cuando el sistema borre la cache
        /// Method called when the systen deletes the cache
        /// </summary>
        /// <param name="clave"></param>
        /// <param name="obj"></param>
        /// <param name="reason"></param>
        private static void SiteMapCacheDeleted(string clave, object obj, CacheItemRemovedReason reason)
        {
            SiteMapConfiguration config
               = (SiteMapConfiguration)ConfigurationManager.GetSection("SubgurimSiteMapConfiguration");


            string path = HttpContext.Current.Server.MapPath(config.dataPath);

            //saveSiteMapXML((Dictionary<string, SiteMap>)obj);
        }*/
        #endregion

        #region SiteMapXML
        /// <summary>
        /// Devuelve el SiteMap a partir del XML
        /// Return the Sitemap from it's XML
        /// </summary>
        /// <returns></returns>
        private static Dictionary<string, SubgurimSiteMapBase> getSiteMapXML()
        {
            // Create the XML if it doesn't exist
            SubgurimSiteMapConfiguration config
                = (SubgurimSiteMapConfiguration)ConfigurationManager.GetSection("SubgurimSiteMapConfiguration");
            string path = HttpContext.Current.Server.MapPath(config.dataPath);

            Dictionary<string, SubgurimSiteMapBase> sitemaps = new Dictionary<string, SubgurimSiteMapBase>();

            string url = path;
            url = url.EndsWith("\\") ? url : url + "\\";
            url += "subgurimsitemap.xml";

            if (!File.Exists(url))
                return sitemaps;

            try
            {
                using (XmlTextReader reader = new XmlTextReader(url))
                {
                    reader.MoveToContent();
                    reader.ReadStartElement();
                    while (reader.Read())
                    {
                        if ((reader.NodeType == XmlNodeType.Element) && (reader.Name == "url"))
                        {
                            getSiteMapXML_Aux(reader.ReadSubtree(), ref sitemaps);
                        }
                    }
                    reader.Close();
                }
            }
            catch
            {
                return new Dictionary<string, SubgurimSiteMapBase>();
            }

            return sitemaps;
        }

        private static void getSiteMapXML_Aux(XmlReader reader, ref Dictionary<string, SubgurimSiteMapBase> sitemaps)
        {
            SubgurimSiteMapBase sitemap = new SubgurimSiteMapBase();

            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element)
                {
                    switch (reader.Name)
                    {
                        case "loc":
                            sitemap.loc = reader.ReadElementContentAsString();
                            break;
                        case "lastmod":
                            sitemap.lastmod = Convert.ToDateTime(reader.ReadString());
                            break;
                        case "changefreq":
                            sitemap.changefreq = (SubgurimSiteMapBase.freq)reader.ReadElementContentAsInt();
                            break;
                        case "priority":
                            sitemap.priority = reader.ReadElementContentAsDouble();
                            break;
                        case "hash":
                            sitemap.hash = reader.ReadElementContentAsString();
                            break;
                        case "lastmodlist":
                            sitemap.lastmodList = getSiteMapXML_lasmodlist(reader.ReadSubtree());
                            break;
                    }
                }
            }
            reader.Close();

            sitemaps.Add(sitemap.loc, sitemap);
        }

        private static List<DateTime> getSiteMapXML_lasmodlist(XmlReader reader)
        {
            List<DateTime> lasmodlist = new List<DateTime>();

            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element)
                {
                    switch (reader.Name)
                    {
                        case "lastmodlistitem":
                            lasmodlist.Add(Convert.ToDateTime(reader.ReadString()));
                            break;
                    }
                }
            }
            reader.Close();

            return lasmodlist;
        }


        /// <summary>
        /// Guarda el XML a partir de un diccionario
        /// Save a XML from the dictionary
        /// </summary>
        /// <param name="SiteMapCache"></param>
        private static void saveSiteMapXML()
        {
            SubgurimSiteMapConfiguration config
                = (SubgurimSiteMapConfiguration)ConfigurationManager.GetSection("SubgurimSiteMapConfiguration");

            string path = HttpContext.Current.Server.MapPath(config.dataPath); ;

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            string url = path;
            url = url.EndsWith("\\") ? url : url + "\\";
            url += "subgurimsitemap.xml";


            try
            {
                using (XmlTextWriter writer = new XmlTextWriter(url, Encoding.UTF8))
                {
                    writer.Formatting = Formatting.Indented;

                    writer.WriteStartDocument();
                    writer.WriteStartElement("SubgurimSiteMap");
                    writer.WriteStartElement("urls");

                    foreach (KeyValuePair<string, SubgurimSiteMapBase> kvp in SiteMaps)
                    {
                        SubgurimSiteMapBase _sitemap;
                        _sitemap = kvp.Value;

                        writer.WriteStartElement("url");

                        writer.WriteElementString("loc", _sitemap.loc);
                        writer.WriteElementString("lastmod", _sitemap.lastmod.ToString());
                        writer.WriteElementString("changefreq", ((int)_sitemap.changefreq).ToString());
                        writer.WriteElementString("priority", _sitemap.priority.ToString(new System.Globalization.CultureInfo("en-US", false)));
                        writer.WriteElementString("hash", _sitemap.hash);

                        writer.WriteStartElement("lastmodlist");
                        foreach (DateTime lastmodlistitem in _sitemap.lastmodList)
                        {
                            writer.WriteElementString("lastmodlistitem", lastmodlistitem.ToString());
                        }
                        writer.WriteEndElement();

                        writer.WriteEndElement();
                    }

                    writer.WriteEndElement();
                    writer.WriteEndElement();
                    writer.WriteEndDocument();

                    writer.Close();
                }
            }
            catch
            {
            }


        }
        
        #endregion

        #region Mostrar SiteMap

        /// <summary>
        /// Devuelve el fichero SiteMap que Google Entiende a partir de nuestra cache
        /// </summary>
        public static string showSiteMap()
        {
            //StringBuilder sb = new StringBuilder();
            //StringWriter sw = new StringWriter(sb);

            MemoryStream ms = new MemoryStream();
            using (XmlTextWriter objX = new XmlTextWriter(ms, new UTF8Encoding(false)))
            {
                objX.WriteStartDocument();
                objX.WriteStartElement("urlset");
                objX.WriteAttributeString("xmlns", "http://www.google.com/schemas/sitemap/0.84");

                SubgurimSiteMapBase _sitemap;
                foreach (KeyValuePair<string, SubgurimSiteMapBase> kvp in SiteMaps)
                {
                    _sitemap = kvp.Value;
                    if (_sitemap.loc != string.Empty)
                    {
                        objX.WriteStartElement("url");

                        objX.WriteElementString("loc", _sitemap.loc);
                        objX.WriteElementString("priority", _sitemap.priority.ToString(new System.Globalization.CultureInfo("en-US", false)));
                        objX.WriteElementString("lastmod", _sitemap.lastmod.ToString("yyyy-MM-dd"));
                        objX.WriteElementString("changefreq", _sitemap.changefreq.ToString());

                        objX.WriteEndElement();
                    }
                }

                objX.WriteEndElement();
                objX.WriteEndDocument();
            }

            //salvo nel xml la cache sitemap (pu accadere che tra lo stop di iis o riclo non si sincronizzi la cache con l'xml)
            //save the xml sitemap cache (which can happen between the stop iis ricycle or not to synchronize the cache with xml)
            saveSiteMapXML();

            return Encoding.UTF8.GetString(ms.ToArray());
            //return sb.ToString();
        }

        #endregion


        #endregion

        #region Add

        /// <summary>
        /// Aade una URL a nuestro listado
        /// Add a URL to our list
        /// </summary>
        /// <param name="loc"></param>
        public static void addSiteMap(string loc)
        {
            loc = loc.ToLower();

            SubgurimSiteMapBase sitemap;
            // If it already was on the dictorionary
            if (SiteMaps.ContainsKey(loc))
            {
                sitemap = SiteMaps[loc];

                // If the page has been accessed  since less than one day, do nothing
                if (((TimeSpan)(DateTime.UtcNow - sitemap.lastAccess)).TotalDays > 1)
                {
                    sitemap.lastAccess = DateTime.UtcNow;

                    //// If the hash is different, the page has changed
                    //// DOESN'T WORK
                    //string hash = SubgurimSiteMapBase.computeHash(loc);
                    //if (!string.IsNullOrEmpty(hash))
                    //{
                    //    if (hash != sitemap.hash)
                        //{
                            //sitemap.hash = hash;
                            //sitemap.updateLastMod(DateTime.UtcNow);
                            //sitemap.computeVariables();
                        //}

                        SiteMaps[loc] = sitemap;

                        saveSiteMapXML_administra();
                    //}
                    //else
                    //{
                    //    // If the page is not accesible, delete from cache
                    //    SiteMaps.Remove(loc);
                    //}
                }
            }
            // If it wasn't on the dictionary
            else
            {
                sitemap = new SubgurimSiteMapBase(loc);
                //sitemap.executeHash();

                //if (!string.IsNullOrEmpty(sitemap.hash))
                {
                    try
                    {
                        SiteMaps.Add(loc, sitemap);
                        saveSiteMapXML_administra();
                    }
                    catch (Exception ex)
                    {
                        
                        //throw;
                    }
                   
                }
            }
        }

        private static void saveSiteMapXML_administra()
        {
            // Update the XML sometimes (pseudo random)
            if (accessToken && ((TimeSpan)(DateTime.UtcNow - lastXMLized)).TotalMinutes > 5)
            {
                accessToken = false;
                lastXMLized = DateTime.UtcNow;
                saveSiteMapXML();
                accessToken = true;
            }
        }

        #endregion

        #region Delete
        public static void clearSiteMap()
        {
            SiteMaps = new Dictionary<string, SubgurimSiteMapBase>();
            saveSiteMapXML();
        }

        public static void deleteSiteMap(string loc)
        {
            if (SiteMaps.ContainsKey(loc))
            {
                SiteMaps.Remove(loc);
                saveSiteMapXML();
            }
        }
        #endregion
    }
}